#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" pykemon.api
User interaction with this package is done through this file.
"""

from beckett.clients import BaseClient

from .request import CHOICES
from .request import make_request

from .resources import (
    MoveResource, PokemonResource, TypeResource, AbilityResource, EggResource,
    DescriptionResource, SpriteResource, GameResource
)


def get(**kwargs):
    """
    Make a request to the PokeAPI server and return the requested resource
    Resource choices:
    pokedex_id
    pokemon
    pokemon_id
    move_id
    ability_id
    type_id
    egg_id
    description_id
    sprite_id
    game_id
    """
    if len(kwargs.keys()) > 1:
        raise ValueError('Too many arguments. Only pass 1 argument')

    if kwargs.keys()[0] in CHOICES:
        return make_request(kwargs)

    else:
        raise ValueError('An invalid argument was passed')


class V1Client(BaseClient):

    class Meta(BaseClient.Meta):
        name = 'pykemon-v1-client'
        base_url = 'http://pokeapi.co/api/v1'
        resources = (
            MoveResource,
            PokemonResource,
            TypeResource,
            AbilityResource,
            EggResource,
            DescriptionResource,
            SpriteResource,
            GameResource,
        )


""" pykemon.models
This files holds all the class definitions representing resources from PokeAPI.
"""


def buildr(bundle, key):
    " Builds a dict of NAME:URI for each item in the bundle "
    return {f['name']: f['resource_uri'] for f in bundle[key]}


class DateTimeObject(object):

    def __init__(self, bundle):
        self.name = bundle['name']
        self.resource_uri = bundle['resource_uri']
        self.created = bundle['created']
        self.modified = bundle['modified']


class Pokemon(DateTimeObject):
    """
    This class represents a single Pokemon resource
    """

    def __init__(self, bundle):
        super(Pokemon, self).__init__(bundle)
        self.id = bundle['national_id']
        self.abilities = buildr(bundle, 'abilities')
        self.egg_groups = buildr(bundle, 'egg_groups')
        self.evolutions = {
            f['to']: f['resource_uri'] for f in bundle['evolutions']}
        self.descriptions = buildr(bundle, 'descriptions')
        self.moves = buildr(bundle, 'moves')
        self.types = buildr(bundle, 'types')
        self.catch_rate = bundle['catch_rate']
        self.species = bundle['species']
        self.hp = bundle['hp']
        self.attack = bundle['attack']
        self.defense = bundle['defense']
        self.sp_atk = bundle['sp_atk']
        self.sp_def = bundle['sp_def']
        self.speed = bundle['speed']
        self.total = bundle['total']
        self.egg_cycles = bundle['egg_cycles']
        self.ev_yield = bundle['ev_yield']
        self.exp = bundle['exp']
        self.growth_rate = bundle['growth_rate']
        self.height = bundle['height']
        self.weight = bundle['weight']
        self.happiness = bundle['happiness']
        self.male_female_ratio = bundle['male_female_ratio']
        self.sprites = buildr(bundle, 'sprites')

    def __repr__(self):
        return '<Pokemon - %s>' % self.name.capitalize()


class Move(DateTimeObject):
    """
    This class represents a single Move resource
    """

    def __init__(self, bundle):
        super(Move, self).__init__(bundle)
        self.id = bundle['id']
        self.accuracy = bundle['accuracy']
        self.category = bundle['category']
        self.power = bundle['power']
        self.pp = bundle['pp']

    def __repr__(self):
        return '<Move - %s>' % self.name.capitalize()


class Type(DateTimeObject):
    """
    This class represents a single Type Resource
    """

    def __init__(self, bundle):
        super(Type, self).__init__(bundle)
        self.id = bundle['id']
        self.name = bundle['name']
        self.ineffective = buildr(bundle, 'ineffective')
        self.resistance = buildr(bundle, 'resistance')
        self.super_effective = buildr(bundle, 'super_effective')
        self.weakness = buildr(bundle, 'weakness')

    def __repr__(self):
        return '<Type - %s>' % self.name.capitalize()


class Ability(DateTimeObject):
    """
    This class represents a single Ability resource
    """

    def __init__(self, bundle):
        super(Ability, self).__init__(bundle)
        self.id = bundle['id']
        self.description = bundle['description']

    def __repr__(self):
        return '<Ability - %s>' % self.name.capitalize()


class Egg(DateTimeObject):
    """
    This class represents a single Egg group resource
    """

    def __init__(self, bundle):
        super(Egg, self).__init__(bundle)
        self.id = bundle['id']
        self.pokemon = buildr(bundle, 'pokemon')

    def __repr__(self):
        return '<Egg - %s>' % self.name.capitalize()


class Description(DateTimeObject):
    """
    This class represents a single Description resource
    """

    def __init__(self, bundle):
        super(Description, self).__init__(bundle)
        self.id = bundle['id']
        self.description = bundle['description']
        self.pokemon = bundle['pokemon']
        self.games = buildr(bundle, 'games')

    def __repr__(self):
        return '<Description - %s>' % self.name.capitalize()


class Sprite(DateTimeObject):
    """
    This class represents a single Sprite resource
    """

    def __init__(self, bundle):
        super(Sprite, self).__init__(bundle)
        self.id = bundle['id']
        self.pokemon = bundle['pokemon']
        self.image = bundle['image']

    def __repr__(self):
        return '<Sprite - %s>' % self.name.capitalize()


class Game(DateTimeObject):
    """
    This class represents a single Game resource
    """

    def __init__(self, bundle):
        super(Game, self).__init__(bundle)
        self.id = bundle['id']
        self.generation = bundle['generation']
        self.release_year = bundle['release_year']

    def __repr__(self):
        return '<Game - %s>' % self.name.capitalize()


""" pykemon.request
This is the request factory for pykemon
All API calls made to the PokeAPI website go from here.
"""

BASE_URI = 'http://pokeapi.co/api/v1'

CHOICES = ['pokedex', 'pokedex_id', 'pokemon', 'pokemon_id', 'move', 'move_id',
           'ability', 'ability_id', 'type', 'type_id', 'egg',
           'egg_id', 'description', 'description_id', 'sprite',
           'sprite_id', 'game', 'game_id']

import requests
import json
from jsons import JSONDecodeError
from .models import Pokemon, Move, Type, Ability, Egg, Description, Sprite, Game
from .exceptions import ResourceNotFoundError

CLASSES = {
    'pokemon': Pokemon,
    'move': Move,
    'type': Type,
    'ability': Ability,
    'egg': Egg,
    'description': Description,
    'sprite': Sprite,
    'game': Game
}


def _request(uri):
    """
    Just a wrapper around the request.get() function
    """

    r = requests.get(uri)

    if r.status_code == 200:
        return _to_json(r.text)
    else:
        raise ResourceNotFoundError(
            'API responded with %s error' % str(r.status_code))


def _to_json(data):
    try:
        content = json.loads(data)
        return content
    except JSONDecodeError:
        raise JSONDecodeError('Error decoding data', data, 0)


def _compose(choice):
    """
    Figure out exactly what resource we're requesting and return the correct
    class.
    """
    nchoice = choice.keys()[0]
    id = choice.values()[0]

    if '_id' in nchoice:
        nchoice = nchoice[:-3]
    return ('/'.join([BASE_URI, nchoice, str(id), '']), nchoice)


def make_request(choice):
    """
    The entry point from pykemon.api.
    Call _request and _compose to figure out the resource / class
    and return the correct constructed object
    """
    uri, nchoice = _compose(choice)
    data = _request(uri)

    resource = CLASSES[nchoice]
    return resource(data)


import os
import sys


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()

setup(
    name='pykemon2.0',
    version='1.0.2',
    description='A Python wrapper for PokeAPI',
    long_description=readme,
    author='JustMaffie',
    author_email='jorivanee@hotmail.com',
    url='https://github.com/justmaffie/pykemon2.0',
    packages=[
        'pykemon2.0',
    ],
    package_dir={'pykemon2.0': 'pykemon2.0'},
    include_package_data=True,
    install_requires=[
        'requests==2.10.0', 'beckett==0.4.0'
    ],
    license="BSD",
    zip_safe=False,
    keywords='pykemon',
    test_suite='tests',
)


__author__ = 'Jori van Ee'
__email__ = 'jorivanee@hotmail.com'
__version__ = '0.0.2'
__license__ = 'BSD'

from .api import get, V1Client  # NOQA
from .exceptions import ResourceNotFoundError  # NOQA


"""
========
Pykemon
========
A Python wrapper for PokeAPI (http://pokeapi.co)
Usage:
>>> import pykemon
>>> pykemon.get(pokemon='bulbasaur')
<Pokemon - Bulbasaur>
>>> pykemon.get(pokemon_id=151)
<Pokemon - Mew>
"""


from beckett.resources import BaseResource


class PokemonResource(BaseResource):
    class Meta(BaseResource.Meta):
        name = 'Pokemon'
        resource_name = 'pokemon'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'national_id',
            'abilities',
            'egg_groups',
            'evolutions',
            'descriptions',
            'moves',
            'types',
            'catch_rate',
            'species',
            'hp',
            'attack',
            'defense',
            'name',
            'sp_atk',
            'sp_def',
            'speed',
            'total',
            'egg_cycles',
            'ev_yield',
            'exp',
            'growth_rate',
            'height',
            'weight',
            'happiness',
            'male_female_ratio',
            'sprites',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class MoveResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Move'
        resource_name = 'move'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'accuracy',
            'category',
            'power',
            'pp',
            'name',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class TypeResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Type'
        resource_name = 'type'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'name',
            'ineffective',
            'resistance',
            'super_effective',
            'weakness',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class AbilityResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Ability'
        resource_name = 'ability'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'name',
            'description',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class EggResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Egg'
        resource_name = 'egg'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'name',
            'pokemon',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class DescriptionResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Description'
        resource_name = 'description'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'name',
            'description',
            'pokemon',
            'games',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class SpriteResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Sprite'
        resource_name = 'sprite'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'name',
            'pokemon',
            'image',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)


class GameResource(BaseResource):

    class Meta(BaseResource.Meta):
        name = 'Game'
        resource_name = 'game'
        identifier = 'id'
        methods = (
            'get',
        )
        attributes = (
            'created',
            'modified',
            'id',
            'name',
            'generation',
            'release_year',
        )

    @staticmethod
    def get_single_resource_url(url, uid, **kwargs):
        # Needs a slash on the end!
        return '{}/{}/'.format(url, uid)